import winston, { format } from "winston"
import DailyRotateFile from "winston-daily-rotate-file"

const isProd = process.env.NODE_ENV !== "dev"

const transports: any[] = [
  new winston.transports.Console({
    level: isProd ? "info" : "silly",
  }),
]

if (isProd) {
  const config = {
    datePattern: "YYYY-MM-DD",
    zippedArchive: true,
    maxSize: "20m",
    maxFiles: "14d",
  }
  transports.push(
    new DailyRotateFile({
      level: "debug",
      filename:
        process.env.LOG_FILE_NAME_DEBUG ??
        "/home/work/log/egg-debug-%DATE%.log",
      ...config,
    })
  )

  transports.push(
    new DailyRotateFile({
      level: "silly",
      filename:
        process.env.LOG_FILE_NAME_SILLY ??
        "/home/work/log/egg-silly-%DATE%.log",
      ...config,
    })
  )
}

const formatList = [
  format.simple(), // 简单文本格式化
  format.timestamp({ format: "YYYY-MM-DD HH:mm:ss" }),
  format.printf(({ level, message, timestamp, requestId }) => {
    const singleLineMessage = isProd
      ? message.replace(/\n/g, " ") // 将换行符替换为空格
      : message
    return `${timestamp} [${level}]${requestId ? ` [RequestId: ${requestId}]` : ""}: ${singleLineMessage}`
  }),
]

// 开发环境下输出彩色日志
if (!isProd) {
  formatList.unshift(format.colorize())
}

const logger = winston.createLogger({
  level: "silly",
  format: format.combine.apply(null, formatList),
  transports,
})

export default logger
